@@ -16,7 +16,8 @@ class O2MTest(_base.MappedTest):
     def define_tables(cls, metadata):
         global foo, bar, blub
         foo = Table('foo', metadata,
-            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
+            Column('id', Integer, primary_key=True,
+                            test_needs_autoincrement=True),
             Column('data', String(20)))
 
         bar = Table('bar', metadata,
@@ -60,14 +61,73 @@ class O2MTest(_base.MappedTest):
         b1.parent_foo = f
         b2.parent_foo = f
         sess.flush()
-        compare = ','.join([repr(b1), repr(b2), repr(b1.parent_foo), repr(b2.parent_foo)])
+        compare = ','.join([repr(b1), repr(b2), repr(b1.parent_foo),
+                           repr(b2.parent_foo)])
         sess.expunge_all()
         l = sess.query(Blub).all()
-        result = ','.join([repr(l[0]), repr(l[1]), repr(l[0].parent_foo), repr(l[1].parent_foo)])
+        result = ','.join([repr(l[0]), repr(l[1]),
+                          repr(l[0].parent_foo), repr(l[1].parent_foo)])
         print compare
         print result
         self.assert_(compare == result)
-        self.assert_(l[0].parent_foo.data == 'foo #1' and l[1].parent_foo.data == 'foo #1')
+        self.assert_(l[0].parent_foo.data == 'foo #1'
+                     and l[1].parent_foo.data == 'foo #1')
+
+class PolymorphicOnNotLocalTest(_base.MappedTest):
+    @classmethod
+    def define_tables(cls, metadata):
+        t1 = Table('t1', metadata, 
+                Column('id', Integer, primary_key=True), 
+                Column('x', String(10)),
+                Column('q', String(10)))
+        t2 = Table('t2', metadata, 
+                Column('id', Integer, primary_key=True), 
+                Column('y', String(10)), 
+                Column('xid', ForeignKey('t1.id')))
+    
+    @testing.resolve_artifact_names
+    def test_bad_polymorphic_on(self):
+        class InterfaceBase(object):
+            pass
+
+        t1t2_join = select([t1.c.x], from_obj=[t1.join(t2)]).alias()
+        def go():
+            interface_m = mapper(InterfaceBase, t2,
+                                polymorphic_on=t1t2_join.c.x,
+                                polymorphic_identity=0)
+
+        assert_raises_message(
+            sa_exc.SAWarning,
+            "Could not map polymorphic_on column 'x' to the mapped table - "
+            "polymorphic loads will not function properly",
+            go
+        )
+        clear_mappers()
+
+        # if its in the with_polymorphic, then its OK
+        interface_m = mapper(InterfaceBase, t2,
+                                polymorphic_on=t1t2_join.c.x,
+                                with_polymorphic=('*', t1t2_join),
+                                polymorphic_identity=0)
+        compile_mappers()
+
+        clear_mappers()
+
+        # if with_polymorphic, but its not present, not OK
+        def go():
+            t1t2_join_2 = select([t1.c.q], from_obj=[t1.join(t2)]).alias()
+            interface_m = mapper(InterfaceBase, t2,
+                                polymorphic_on=t1t2_join.c.x,
+                                with_polymorphic=('*', t1t2_join_2),
+                                polymorphic_identity=0)
+        assert_raises_message(
+            sa_exc.SAWarning,
+            "Could not map polymorphic_on column 'x' to the mapped table - "
+            "polymorphic loads will not function properly",
+            go
+        )
+        
+        
 
 class FalseDiscriminatorTest(_base.MappedTest):
     @classmethod
@@ -108,11 +168,13 @@ class PolymorphicSynonymTest(_base.MappedTest):
     def define_tables(cls, metadata):
         global t1, t2
         t1 = Table('t1', metadata,
-                   Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
+                   Column('id', Integer, primary_key=True,
+                                    test_needs_autoincrement=True),
                    Column('type', String(10), nullable=False),
                    Column('info', String(255)))
         t2 = Table('t2', metadata,
-                   Column('id', Integer, ForeignKey('t1.id'), primary_key=True),
+                   Column('id', Integer, ForeignKey('t1.id'),
+                                            primary_key=True),
                    Column('data', String(10), nullable=False))
     
     def test_polymorphic_synonym(self):
@@ -125,9 +187,10 @@ class PolymorphicSynonymTest(_base.MappedTest):
             
         class T2(T1):pass
         
-        mapper(T1, t1, polymorphic_on=t1.c.type, polymorphic_identity='t1', properties={
-            'info':synonym('_info', map_column=True)
-        })
+        mapper(T1, t1, polymorphic_on=t1.c.type, polymorphic_identity='t1',
+            properties={
+                'info':synonym('_info', map_column=True)
+            })
         mapper(T2, t2, inherits=T1, polymorphic_identity='t2')
         sess = create_session()
         at1 = T1(info='at1')
@@ -605,14 +668,14 @@ class VersioningTest(_base.MappedTest):
 
         sess.flush()
 
-        assert_raises(orm_exc.ConcurrentModificationError,
+        assert_raises(orm_exc.StaleDataError,
                         sess2.query(Base).with_lockmode('read').get, 
                         s1.id)
 
         if not testing.db.dialect.supports_sane_rowcount:
             sess2.flush()
         else:
-            assert_raises(orm_exc.ConcurrentModificationError, sess2.flush)
+            assert_raises(orm_exc.StaleDataError, sess2.flush)
 
         sess2.refresh(s2)
         if testing.db.dialect.supports_sane_rowcount:
@@ -652,12 +715,14 @@ class VersioningTest(_base.MappedTest):
         s2.subdata = 'some new subdata'
         sess.flush()
 
-        try:
-            s1.subdata = 'some new subdata'
+        s1.subdata = 'some new subdata'
+        if testing.db.dialect.supports_sane_rowcount:
+            assert_raises(
+                orm_exc.StaleDataError,
+                sess.flush
+            )
+        else:
             sess.flush()
-            assert not testing.db.dialect.supports_sane_rowcount
-        except orm_exc.ConcurrentModificationError, e:
-            assert True
 
 class DistinctPKTest(_base.MappedTest):
     """test the construction of mapper.primary_key when an inheriting relationship
@@ -916,7 +981,8 @@ class OverrideColKeyTest(_base.MappedTest):
         # s2 gets a new id, base_id is overwritten by the ultimate
         # PK col
         assert s2.id == s2.base_id != 15
-        
+    
+    @testing.emits_warning(r'Implicit')
     def test_override_implicit(self):
         # this is how the pattern looks intuitively when 
         # using declarative.
@@ -1078,7 +1144,9 @@ class OptimizedLoadTest(_base.MappedTest):
         
         # redefine Sub's "id" to favor the "id" col in the subtable.
         # "id" is also part of the primary join condition
-        mapper(Sub, sub, inherits=Base, polymorphic_identity='sub', properties={'id':sub.c.id})
+        mapper(Sub, sub, inherits=Base, 
+                        polymorphic_identity='sub',
+                        properties={'id':[sub.c.id, base.c.id]})
         sess = sessionmaker()()
         s1 = Sub(data='s1data', sub='s1sub')
         sess.add(s1)
