@@ -11,6 +11,7 @@ from sqlalchemy.test import TestBase, AssertsExecutionResults, testing
 from sqlalchemy.util import function_named
 from test.orm import _base, _fixtures
 from sqlalchemy.test.testing import eq_
+from sqlalchemy.test.schema import Table, Column
 
 class AttrSettable(object):
     def __init__(self, **kwargs):
@@ -19,7 +20,7 @@ class AttrSettable(object):
         return self.__class__.__name__ + "(%s)" % (hex(id(self)))
 
 
-class RelationTest1(_base.MappedTest):
+class RelationshipTest1(_base.MappedTest):
     """test self-referential relationships on polymorphic mappers"""
     @classmethod
     def define_tables(cls, metadata):
@@ -39,7 +40,7 @@ class RelationTest1(_base.MappedTest):
 
     def teardown(self):
         people.update(values={people.c.manager_id:None}).execute()
-        super(RelationTest1, self).teardown()
+        super(RelationshipTest1, self).teardown()
 
     def test_parent_refs_descendant(self):
         class Person(AttrSettable):
@@ -50,7 +51,7 @@ class RelationTest1(_base.MappedTest):
         # note that up until recently (0.4.4), we had to specify "foreign_keys" here
         # for this primary join.  
         mapper(Person, people, properties={
-            'manager':relation(Manager, primaryjoin=(people.c.manager_id ==
+            'manager':relationship(Manager, primaryjoin=(people.c.manager_id ==
                                                      managers.c.person_id),
                                uselist=False, post_update=True)
         })
@@ -80,7 +81,7 @@ class RelationTest1(_base.MappedTest):
 
         mapper(Person, people)
         mapper(Manager, managers, inherits=Person, inherit_condition=people.c.person_id==managers.c.person_id, properties={
-            'employee':relation(Person, primaryjoin=(people.c.manager_id ==
+            'employee':relationship(Person, primaryjoin=(people.c.manager_id ==
                                                      managers.c.person_id),
                                 foreign_keys=[people.c.manager_id],
                                 uselist=False, post_update=True)
@@ -99,13 +100,13 @@ class RelationTest1(_base.MappedTest):
         print p, m, m.employee
         assert m.employee is p
 
-class RelationTest2(_base.MappedTest):
+class RelationshipTest2(_base.MappedTest):
     """test self-referential relationships on polymorphic mappers"""
     @classmethod
     def define_tables(cls, metadata):
         global people, managers, data
         people = Table('people', metadata,
-           Column('person_id', Integer, Sequence('person_id_seq', optional=True), primary_key=True),
+           Column('person_id', Integer, primary_key=True, test_needs_autoincrement=True),
            Column('name', String(50)),
            Column('type', String(30)))
 
@@ -120,17 +121,17 @@ class RelationTest2(_base.MappedTest):
             Column('data', String(30))
             )
 
-    def testrelationonsubclass_j1_nodata(self):
+    def testrelationshiponsubclass_j1_nodata(self):
         self.do_test("join1", False)
-    def testrelationonsubclass_j2_nodata(self):
+    def testrelationshiponsubclass_j2_nodata(self):
         self.do_test("join2", False)
-    def testrelationonsubclass_j1_data(self):
+    def testrelationshiponsubclass_j1_data(self):
         self.do_test("join1", True)
-    def testrelationonsubclass_j2_data(self):
+    def testrelationshiponsubclass_j2_data(self):
         self.do_test("join2", True)
-    def testrelationonsubclass_j3_nodata(self):
+    def testrelationshiponsubclass_j3_nodata(self):
         self.do_test("join3", False)
-    def testrelationonsubclass_j3_data(self):
+    def testrelationshiponsubclass_j3_data(self):
         self.do_test("join3", True)
 
     def do_test(self, jointype="join1", usedata=False):
@@ -166,14 +167,14 @@ class RelationTest2(_base.MappedTest):
         if usedata:
             mapper(Manager, managers, inherits=Person, inherit_condition=people.c.person_id==managers.c.person_id, polymorphic_identity='manager',
                   properties={
-                    'colleague':relation(Person, primaryjoin=managers.c.manager_id==people.c.person_id, lazy=True, uselist=False),
-                    'data':relation(Data, uselist=False)
+                    'colleague':relationship(Person, primaryjoin=managers.c.manager_id==people.c.person_id, lazy='select', uselist=False),
+                    'data':relationship(Data, uselist=False)
                  }
             )
         else:
             mapper(Manager, managers, inherits=Person, inherit_condition=people.c.person_id==managers.c.person_id, polymorphic_identity='manager',
                   properties={
-                    'colleague':relation(Person, primaryjoin=managers.c.manager_id==people.c.person_id, lazy=True, uselist=False)
+                    'colleague':relationship(Person, primaryjoin=managers.c.manager_id==people.c.person_id, lazy='select', uselist=False)
                  }
             )
 
@@ -195,13 +196,13 @@ class RelationTest2(_base.MappedTest):
         if usedata:
             assert m.data.data == 'ms data'
 
-class RelationTest3(_base.MappedTest):
+class RelationshipTest3(_base.MappedTest):
     """test self-referential relationships on polymorphic mappers"""
     @classmethod
     def define_tables(cls, metadata):
         global people, managers, data
         people = Table('people', metadata,
-           Column('person_id', Integer, Sequence('person_id_seq', optional=True), primary_key=True),
+           Column('person_id', Integer, primary_key=True, test_needs_autoincrement=True),
            Column('colleague_id', Integer, ForeignKey('people.person_id')),
            Column('name', String(50)),
            Column('type', String(30)))
@@ -249,14 +250,14 @@ def _generate_test(jointype="join1", usedata=False):
         if usedata:
             mapper(Person, people, with_polymorphic=('*', poly_union), polymorphic_identity='person', polymorphic_on=people.c.type,
                   properties={
-                    'colleagues':relation(Person, primaryjoin=people.c.colleague_id==people.c.person_id, remote_side=people.c.colleague_id, uselist=True),
-                    'data':relation(Data, uselist=False)
+                    'colleagues':relationship(Person, primaryjoin=people.c.colleague_id==people.c.person_id, remote_side=people.c.colleague_id, uselist=True),
+                    'data':relationship(Data, uselist=False)
                     }
             )
         else:
             mapper(Person, people, with_polymorphic=('*', poly_union), polymorphic_identity='person', polymorphic_on=people.c.type,
                   properties={
-                    'colleagues':relation(Person, primaryjoin=people.c.colleague_id==people.c.person_id,
+                    'colleagues':relationship(Person, primaryjoin=people.c.colleague_id==people.c.person_id,
                         remote_side=people.c.colleague_id, uselist=True)
                     }
             )
@@ -292,22 +293,22 @@ def _generate_test(jointype="join1", usedata=False):
             assert m.data.data == 'ms data'
 
     do_test = function_named(
-        do_test, 'test_relation_on_base_class_%s_%s' % (
+        do_test, 'test_relationship_on_base_class_%s_%s' % (
         jointype, data and "nodata" or "data"))
     return do_test
 
 for jointype in ["join1", "join2", "join3", "join4"]:
     for data in (True, False):
         func = _generate_test(jointype, data)
-        setattr(RelationTest3, func.__name__, func)
+        setattr(RelationshipTest3, func.__name__, func)
 del func
 
-class RelationTest4(_base.MappedTest):
+class RelationshipTest4(_base.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         global people, engineers, managers, cars
         people = Table('people', metadata,
-           Column('person_id', Integer, primary_key=True),
+           Column('person_id', Integer, primary_key=True, test_needs_autoincrement=True),
            Column('name', String(50)))
 
         engineers = Table('engineers', metadata,
@@ -319,7 +320,7 @@ class RelationTest4(_base.MappedTest):
            Column('longer_status', String(70)))
 
         cars = Table('cars', metadata,
-           Column('car_id', Integer, primary_key=True),
+           Column('car_id', Integer, primary_key=True, test_needs_autoincrement=True),
            Column('owner', Integer, ForeignKey('people.person_id')))
 
     def testmanytoonepolymorphic(self):
@@ -358,7 +359,7 @@ class RelationTest4(_base.MappedTest):
         person_mapper   = mapper(Person, people, with_polymorphic=('*', employee_join), polymorphic_on=employee_join.c.type, polymorphic_identity='person')
         engineer_mapper = mapper(Engineer, engineers, inherits=person_mapper, polymorphic_identity='engineer')
         manager_mapper  = mapper(Manager, managers, inherits=person_mapper, polymorphic_identity='manager')
-        car_mapper      = mapper(Car, cars, properties= {'employee':relation(person_mapper)})
+        car_mapper      = mapper(Car, cars, properties= {'employee':relationship(person_mapper)})
 
         session = create_session()
 
@@ -383,7 +384,7 @@ class RelationTest4(_base.MappedTest):
         session.expunge_all()
     
         def go():
-            testcar = session.query(Car).options(eagerload('employee')).get(car1.car_id)
+            testcar = session.query(Car).options(joinedload('employee')).get(car1.car_id)
             assert str(testcar.employee) == "Engineer E4, status X"
         self.assert_sql_count(testing.db, go, 1)
 
@@ -406,7 +407,7 @@ class RelationTest4(_base.MappedTest):
         # and now for the lightning round, eager !
 
         def go():
-            testcar = session.query(Car).options(eagerload('employee')).get(car1.car_id)
+            testcar = session.query(Car).options(joinedload('employee')).get(car1.car_id)
             assert str(testcar.employee) == "Engineer E4, status X"
         self.assert_sql_count(testing.db, go, 1)
 
@@ -415,12 +416,12 @@ class RelationTest4(_base.MappedTest):
         c = s.join("employee").filter(Person.name=="E4")[0]
         assert c.car_id==car1.car_id
 
-class RelationTest5(_base.MappedTest):
+class RelationshipTest5(_base.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         global people, engineers, managers, cars
         people = Table('people', metadata,
-           Column('person_id', Integer, primary_key=True),
+           Column('person_id', Integer, primary_key=True, test_needs_autoincrement=True),
            Column('name', String(50)),
            Column('type', String(50)))
 
@@ -433,11 +434,11 @@ class RelationTest5(_base.MappedTest):
            Column('longer_status', String(70)))
 
         cars = Table('cars', metadata,
-           Column('car_id', Integer, primary_key=True),
+           Column('car_id', Integer, primary_key=True, test_needs_autoincrement=True),
            Column('owner', Integer, ForeignKey('people.person_id')))
 
-    def testeagerempty(self):
-        """an easy one...test parent object with child relation to an inheriting mapper, using eager loads,
+    def test_eager_empty(self):
+        """test parent object with child relationship to an inheriting mapper, using eager loads,
         works when there are no child objects present"""
         class Person(object):
             def __init__(self, **kwargs):
@@ -461,7 +462,7 @@ class RelationTest5(_base.MappedTest):
         person_mapper   = mapper(Person, people, polymorphic_on=people.c.type, polymorphic_identity='person')
         engineer_mapper = mapper(Engineer, engineers, inherits=person_mapper, polymorphic_identity='engineer')
         manager_mapper  = mapper(Manager, managers, inherits=person_mapper, polymorphic_identity='manager')
-        car_mapper      = mapper(Car, cars, properties= {'manager':relation(manager_mapper, lazy=False)})
+        car_mapper      = mapper(Car, cars, properties= {'manager':relationship(manager_mapper, lazy='joined')})
 
         sess = create_session()
         car1 = Car()
@@ -476,13 +477,13 @@ class RelationTest5(_base.MappedTest):
         assert carlist[0].manager is None
         assert carlist[1].manager.person_id == car2.manager.person_id
 
-class RelationTest6(_base.MappedTest):
+class RelationshipTest6(_base.MappedTest):
     """test self-referential relationships on a single joined-table inheritance mapper"""
     @classmethod
     def define_tables(cls, metadata):
         global people, managers, data
         people = Table('people', metadata,
-           Column('person_id', Integer, Sequence('person_id_seq', optional=True), primary_key=True),
+           Column('person_id', Integer, primary_key=True, test_needs_autoincrement=True),
            Column('name', String(50)),
            )
 
@@ -504,7 +505,7 @@ class RelationTest6(_base.MappedTest):
         # to parent.mapped_table
         mapper(Manager, managers, inherits=Person, inherit_condition=people.c.person_id==managers.c.person_id,
               properties={
-                'colleague':relation(Manager, primaryjoin=managers.c.colleague_id==managers.c.person_id, lazy=True, uselist=False)
+                'colleague':relationship(Manager, primaryjoin=managers.c.colleague_id==managers.c.person_id, lazy='select', uselist=False)
              }
         )
 
@@ -520,19 +521,19 @@ class RelationTest6(_base.MappedTest):
         m2 = sess.query(Manager).get(m2.person_id)
         assert m.colleague is m2
 
-class RelationTest7(_base.MappedTest):
+class RelationshipTest7(_base.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         global people, engineers, managers, cars, offroad_cars
         cars = Table('cars', metadata,
-                Column('car_id', Integer, primary_key=True),
+                Column('car_id', Integer, primary_key=True, test_needs_autoincrement=True),
                 Column('name', String(30)))
 
         offroad_cars = Table('offroad_cars', metadata,
                 Column('car_id',Integer, ForeignKey('cars.car_id'),nullable=False,primary_key=True))
 
         people = Table('people', metadata,
-                Column('person_id', Integer, primary_key=True),
+                Column('person_id', Integer, primary_key=True, test_needs_autoincrement=True),
                 Column('car_id', Integer, ForeignKey('cars.car_id'), nullable=False),
                 Column('name', String(50)))
 
@@ -545,6 +546,7 @@ class RelationTest7(_base.MappedTest):
                 Column('person_id', Integer, ForeignKey('people.person_id'), primary_key=True),
                 Column('category', String(70)))
 
+    @testing.uses_deprecated("fold_equivalents is deprecated.")
     def test_manytoone_lazyload(self):
         """test that lazy load clause to a polymorphic child mapper generates correctly [ticket:493]"""
         class PersistentObject(object):
@@ -597,7 +599,7 @@ class RelationTest7(_base.MappedTest):
                 with_polymorphic=('*', employee_join), polymorphic_on=employee_join.c.type,
                 polymorphic_identity='person',
                 properties={
-                    'car':relation(car_mapper)
+                    'car':relationship(car_mapper)
                     })
         engineer_mapper = mapper(Engineer, engineers, inherits=person_mapper, polymorphic_identity='engineer')
         manager_mapper  = mapper(Manager, managers, inherits=person_mapper, polymorphic_identity='manager')
@@ -620,12 +622,12 @@ class RelationTest7(_base.MappedTest):
         for p in r:
             assert p.car_id == p.car.car_id
 
-class RelationTest8(_base.MappedTest):
+class RelationshipTest8(_base.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         global taggable, users
         taggable = Table('taggable', metadata,
-                         Column('id', Integer, primary_key=True),
+                         Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
                          Column('type', String(30)),
                          Column('owner_id', Integer, ForeignKey('taggable.id')),
                          )
@@ -642,7 +644,7 @@ class RelationTest8(_base.MappedTest):
             pass
 
         mapper( Taggable, taggable, polymorphic_on=taggable.c.type, polymorphic_identity='taggable', properties = {
-            'owner' : relation (User,
+            'owner' : relationship (User,
                                primaryjoin=taggable.c.owner_id ==taggable.c.id,
                                 remote_side=taggable.c.id
                                 ),
@@ -680,11 +682,11 @@ class GenerativeTest(TestBase, AssertsExecutionResults):
         metadata = MetaData(testing.db)
         # table definitions
         status = Table('status', metadata,
-           Column('status_id', Integer, primary_key=True),
+           Column('status_id', Integer, primary_key=True, test_needs_autoincrement=True),
            Column('name', String(20)))
 
         people = Table('people', metadata,
-           Column('person_id', Integer, primary_key=True),
+           Column('person_id', Integer, primary_key=True, test_needs_autoincrement=True),
            Column('status_id', Integer, ForeignKey('status.status_id'), nullable=False),
            Column('name', String(50)))
 
@@ -697,7 +699,7 @@ class GenerativeTest(TestBase, AssertsExecutionResults):
            Column('category', String(70)))
 
         cars = Table('cars', metadata,
-           Column('car_id', Integer, primary_key=True),
+           Column('car_id', Integer, primary_key=True, test_needs_autoincrement=True),
            Column('status_id', Integer, ForeignKey('status.status_id'), nullable=False),
            Column('owner', Integer, ForeignKey('people.person_id'), nullable=False))
 
@@ -743,10 +745,10 @@ class GenerativeTest(TestBase, AssertsExecutionResults):
         status_mapper   = mapper(Status, status)
         person_mapper   = mapper(Person, people,
             with_polymorphic=('*', employee_join), polymorphic_on=employee_join.c.type,
-            polymorphic_identity='person', properties={'status':relation(status_mapper)})
+            polymorphic_identity='person', properties={'status':relationship(status_mapper)})
         engineer_mapper = mapper(Engineer, engineers, inherits=person_mapper, polymorphic_identity='engineer')
         manager_mapper  = mapper(Manager, managers, inherits=person_mapper, polymorphic_identity='manager')
-        car_mapper      = mapper(Car, cars, properties= {'employee':relation(person_mapper), 'status':relation(status_mapper)})
+        car_mapper      = mapper(Car, cars, properties= {'employee':relationship(person_mapper), 'status':relationship(status_mapper)})
 
         session = create_session()
 
@@ -786,13 +788,13 @@ class GenerativeTest(TestBase, AssertsExecutionResults):
         e = exists([Car.owner], Car.owner==employee_join.c.person_id)
         Query(Person)._adapt_clause(employee_join, False, False)
         
-        r = session.query(Person).filter(Person.name.like('%2')).join('status').filter_by(name="active")
-        assert str(list(r)) == "[Manager M2, category YYYYYYYYY, status Status active, Engineer E2, field X, status Status active]"
+        r = session.query(Person).filter(Person.name.like('%2')).join('status').filter_by(name="active").order_by(Person.person_id)
+        eq_(str(list(r)), "[Manager M2, category YYYYYYYYY, status Status active, Engineer E2, field X, status Status active]")
         r = session.query(Engineer).join('status').filter(Person.name.in_(['E2', 'E3', 'E4', 'M4', 'M2', 'M1']) & (status.c.name=="active")).order_by(Person.name)
-        assert str(list(r)) == "[Engineer E2, field X, status Status active, Engineer E3, field X, status Status active]"
+        eq_(str(list(r)), "[Engineer E2, field X, status Status active, Engineer E3, field X, status Status active]")
 
         r = session.query(Person).filter(exists([1], Car.owner==Person.person_id))
-        assert str(list(r)) == "[Engineer E4, field X, status Status dead]"
+        eq_(str(list(r)), "[Engineer E4, field X, status Status dead]")
 
 class MultiLevelTest(_base.MappedTest):
     @classmethod
@@ -800,7 +802,7 @@ class MultiLevelTest(_base.MappedTest):
         global table_Employee, table_Engineer, table_Manager
         table_Employee = Table( 'Employee', metadata,
             Column( 'name', type_= String(100), ),
-            Column( 'id', primary_key= True, type_= Integer, ),
+            Column( 'id', primary_key= True, type_= Integer, test_needs_autoincrement=True),
             Column( 'atype', type_= String(100), ),
         )
 
@@ -878,7 +880,7 @@ class ManyToManyPolyTest(_base.MappedTest):
         global base_item_table, item_table, base_item_collection_table, collection_table
         base_item_table = Table(
             'base_item', metadata,
-            Column('id', Integer, primary_key=True),
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('child_name', String(255), default=None))
 
         item_table = Table(
@@ -893,7 +895,7 @@ class ManyToManyPolyTest(_base.MappedTest):
 
         collection_table = Table(
             'collection', metadata,
-            Column('id', Integer, primary_key=True),
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('name', Unicode(255)))
 
     def test_pjoin_compile(self):
@@ -912,7 +914,7 @@ class ManyToManyPolyTest(_base.MappedTest):
             with_polymorphic=('*', item_join),
             polymorphic_on=base_item_table.c.child_name,
             polymorphic_identity='BaseItem',
-            properties=dict(collections=relation(Collection, secondary=base_item_collection_table, backref="items")))
+            properties=dict(collections=relationship(Collection, secondary=base_item_collection_table, backref="items")))
 
         mapper(
             Item, item_table,
@@ -928,7 +930,7 @@ class CustomPKTest(_base.MappedTest):
     def define_tables(cls, metadata):
         global t1, t2
         t1 = Table('t1', metadata,
-            Column('id', Integer, primary_key=True),
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('type', String(30), nullable=False),
             Column('data', String(30)))
         # note that the primary key column in t2 is named differently
@@ -1013,7 +1015,7 @@ class InheritingEagerTest(_base.MappedTest):
         global people, employees, tags, peopleTags
 
         people = Table('people', metadata,
-                           Column('id', Integer, primary_key=True),
+                           Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
                            Column('_type', String(30), nullable=False),
                           )
 
@@ -1023,7 +1025,7 @@ class InheritingEagerTest(_base.MappedTest):
                         )
 
         tags = Table('tags', metadata,
-                           Column('id', Integer, primary_key=True),
+                           Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
                            Column('label', String(50), nullable=False),
                        )
 
@@ -1047,7 +1049,7 @@ class InheritingEagerTest(_base.MappedTest):
                self.label = label
 
         mapper(Person, people, polymorphic_on=people.c._type,polymorphic_identity='person', properties={
-            'tags': relation(Tag, secondary=peopleTags,backref='people', lazy=False)
+            'tags': relationship(Tag, secondary=peopleTags,backref='people', lazy='joined')
         })
         mapper(Employee, employees, inherits=Person,polymorphic_identity='employee')
         mapper(Tag, tags)
@@ -1074,11 +1076,11 @@ class MissingPolymorphicOnTest(_base.MappedTest):
     def define_tables(cls, metadata):
         global tablea, tableb, tablec, tabled
         tablea = Table('tablea', metadata, 
-            Column('id', Integer, primary_key=True),
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('adata', String(50)),
             )
         tableb = Table('tableb', metadata, 
-            Column('id', Integer, primary_key=True),
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('aid', Integer, ForeignKey('tablea.id')),
             Column('data', String(50)),
             )
@@ -1105,7 +1107,7 @@ class MissingPolymorphicOnTest(_base.MappedTest):
         
         mapper(B, tableb)
         mapper(A, tablea, with_polymorphic=('*', poly_select), polymorphic_on=poly_select.c.discriminator, properties={
-            'b':relation(B, uselist=False)
+            'b':relationship(B, uselist=False)
         })
         mapper(C, tablec, inherits=A,polymorphic_identity='c')
         mapper(D, tabled, inherits=C, polymorphic_identity='d')
